class Category
  include Enumerable

  attr_reader :loaded, :title
  
  # Initialize an empty category
  def initialize(num_cells = 0)
    @cells = (0...num_cells).collect { Cell.new }
    @loaded = 0
    @title = FormattedText.new
  end
  
  def title=(t)
    @title.text = t
  end

  # Resize the category
  def resize(num_cells)
    @cells = (0...num_cells).collect { Cell.new }
    @loaded = 0
  end

  # Access the cells by index
  def [](i)
    @cells[i]
  end

  # Return the size of the board
  def num_cells
    @cells.size
  end
  
  # Allow iteration through the cells
  def each(*args, &block)
    @cells.each(*args, &block)
  end

  # Clear the 'done' flag in the entire category
  def clear
    @cells.each { |cell| cell.clear }
  end
  
  # Access the last loaded cell (for incomplete load)
  def last_cell_loaded
    @cells[@loaded]
  end

  # Access the last loaded cell number (for incomplete load)
  def last_cell_number_loaded
    @loaded + 1
  end

  # Load from a reader
  def load(reader)
    @title.style_size, args = reader.read_keyword('@CATEGORY', true, 1, 'Missing category title after "@CATEGORY".')
    @title.text = args[0]
    @cells.each { |cell| cell.load(reader); @loaded += 1}
  end
end
