class IOReader
  # Initialize using a stream and have a line number
  def initialize(fin)
    @index = 0
    @lines = fin.readlines
  end
  
  # Returns the current line number
  def line_number
    @index
  end
  
  # Peeks to the next non-trivial line and returns it. The line is still available
  # to read in subsequent calls.
  def peekline(comment = '#', allow_blank = false)
    line = ''
    while ( @index < @lines.size )
      line = @lines[@index]
      break if line[0] != ?# and (!allow_blank or line.chomp.size > 0)
      @index += 1
    end

    raise EOFError, "Cannot read past end of file."  if @index >= @lines.size

    line
  end

  # Reads the next non-trivial line.
  def readline(comment = '#', allow_blank = false)
    line = peekline(comment, allow_blank)
    @index += 1
    line
  end

  # Reads and accumulates lines until it reaches a line satisfying the regular
  # expression. This line is not included in the accumulated string and is available
  # to read in subsequent calls.
  def readlines_til(endcase_regexp, comment = '#', allow_blank = false)
    total = ''
    while (line = peekline(comment, allow_blank)) !~ endcase_regexp
      total << line
      @index += 1
    end

    total
  end

  # Returns a string with the lines around the current index joined by the given separator  
  def lines_around_current(buffer = 4, show_line_numbers = true, join_separator = '')
    first = [@index-1-buffer, 0].max
    last  = [@index-1+buffer, @lines.size-1].min
    lines = @lines[first..last]
    if show_line_numbers
      lines = (0...lines.size).collect { |i| "%9d:   %s" % [i+first+1, lines[i]] }
    end

    lines.join(join_separator)
  end
end